@@ -13,14 +13,14 @@ import h5py
 
 from sklearn.metrics import roc_auc_score, f1_score
 
-from cropharvest.eo import STATIC_BANDS, DYNAMIC_BANDS
+from cropharvest.bands import STATIC_BANDS, DYNAMIC_BANDS
 from cropharvest.columns import RequiredColumns, NullableColumns
 from .config import (
     EXPORT_END_DAY,
     EXPORT_END_MONTH,
     LABELS_FILENAME,
     DAYS_PER_TIMESTEP,
-    NUM_TIMESTEPS,
+    DEFAULT_NUM_TIMESTEPS,
     TEST_REGIONS,
     TEST_DATASETS,
 )
@@ -200,11 +200,15 @@ class Engineer:
         return int(index), dataset
 
     @staticmethod
-    def load_tif(filepath: Path, start_date: datetime) -> Tuple[xr.DataArray, float]:
+    def load_tif(
+        filepath: Path, start_date: datetime, num_timesteps: Optional[int] = DEFAULT_NUM_TIMESTEPS
+    ) -> Tuple[xr.DataArray, float]:
         r"""
         The sentinel files exported from google earth have all the timesteps
         concatenated together. This function loads a tif files and splits the
         timesteps
+
+        Returns: The loaded xr.DataArray, and the average slope (used for filling nan slopes)
         """
 
         da = xr.open_rasterio(filepath).rename("FEATURES")
@@ -215,12 +219,14 @@ class Engineer:
         num_bands = len(da.band)
         num_dynamic_bands = num_bands - len(STATIC_BANDS)
 
-        assert num_dynamic_bands == bands_per_timestep * NUM_TIMESTEPS
+        assert num_dynamic_bands % bands_per_timestep == 0
+        if num_timesteps is None:
+            num_timesteps = num_dynamic_bands // bands_per_timestep
 
         static_data = da.isel(band=slice(num_bands - len(STATIC_BANDS), num_bands))
         average_slope = np.nanmean(static_data.values[STATIC_BANDS.index("slope"), :, :])
 
-        for timestep in range(NUM_TIMESTEPS):
+        for timestep in range(num_timesteps):
             time_specific_da = da.isel(
                 band=slice(timestep * bands_per_timestep, (timestep + 1) * bands_per_timestep)
             )
@@ -229,7 +235,8 @@ class Engineer:
             da_split_by_time.append(time_specific_da)
 
         timesteps = [
-            start_date + timedelta(days=DAYS_PER_TIMESTEP) * i for i in range(NUM_TIMESTEPS)
+            start_date + timedelta(days=DAYS_PER_TIMESTEP) * i
+            for i in range(len(da_split_by_time))
         ]
 
         dynamic_data = xr.concat(da_split_by_time, pd.Index(timesteps, name="time"))
@@ -331,6 +338,7 @@ class Engineer:
         r"""
         Given an input array of shape [timesteps, BANDS]
         fill NaN values with the mean of each band across the timestep
+        The slope values may all be nan so average_slope is manually passed
         """
         num_dims = len(array.shape)
         if num_dims == 2:
@@ -388,33 +396,48 @@ class Engineer:
             # Unreachable code logically but mypy does not see it this way
             raise ValueError(error_message)
 
-    def process_test_file(self, path_to_file: Path, id_in_region: int) -> Tuple[str, TestInstance]:
-        id_components = path_to_file.name.split("_")
-        crop, end_year = id_components[1], id_components[2]
-        identifier = "_".join(id_components[:4])
-        identifier_plus_idx = f"{identifier}_{id_in_region}"
-        start_date = datetime(int(end_year), EXPORT_END_MONTH, EXPORT_END_DAY) - timedelta(
-            days=NUM_TIMESTEPS * DAYS_PER_TIMESTEP
-        )
-        da, average_slope = self.load_tif(path_to_file, start_date=start_date)
-        lon, lat = np.meshgrid(da.x.values, da.y.values)
-        flat_lat, flat_lon = (
-            np.squeeze(lat.reshape(-1, 1), -1),
-            np.squeeze(lon.reshape(-1, 1), -1),
-        )
+    @staticmethod
+    def process_test_file(
+        path_to_file: Path, start_date: datetime
+    ) -> Tuple[np.ndarray, np.ndarray, np.ndarray]:
+        da, slope = Engineer.load_tif(path_to_file, start_date=start_date)
 
+        # Process remote sensing data
         x_np = da.values
         x_np = x_np.reshape(x_np.shape[0], x_np.shape[1], x_np.shape[2] * x_np.shape[3])
         x_np = np.moveaxis(x_np, -1, 0)
-        x_np = self.calculate_ndvi(x_np)
-        x_np = self.remove_bands(x_np)
-        final_x = self.fillna(x_np, average_slope)
+        x_np = Engineer.calculate_ndvi(x_np)
+        x_np = Engineer.remove_bands(x_np)
+        final_x = Engineer.fillna(x_np, slope)
         if final_x is None:
             raise RuntimeError(
                 "fillna on the test instance returned None; "
                 "does the test instance contain NaN only bands?"
             )
 
+        # Get lat lons
+        lon, lat = np.meshgrid(da.x.values, da.y.values)
+        flat_lat, flat_lon = (
+            np.squeeze(lat.reshape(-1, 1), -1),
+            np.squeeze(lon.reshape(-1, 1), -1),
+        )
+        return final_x, flat_lat, flat_lon
+
+    def process_test_file_with_region(
+        self, path_to_file: Path, id_in_region: int
+    ) -> Tuple[str, TestInstance]:
+        id_components = path_to_file.name.split("_")
+        crop, end_year = id_components[1], id_components[2]
+        identifier = "_".join(id_components[:4])
+        identifier_plus_idx = f"{identifier}_{id_in_region}"
+        start_date = datetime(int(end_year), EXPORT_END_MONTH, EXPORT_END_DAY) - timedelta(
+            days=DEFAULT_NUM_TIMESTEPS * DAYS_PER_TIMESTEP
+        )
+
+        final_x, flat_lat, flat_lon = Engineer.process_test_file(
+            path_to_file, start_date=start_date
+        )
+
         # finally, we need to calculate the mask
         region_bbox = TEST_REGIONS[identifier]
         relevant_indices = self.labels.apply(
@@ -450,8 +473,13 @@ class Engineer:
 
         return identifier_plus_idx, TestInstance(x=final_x, y=y, lats=flat_lat, lons=flat_lon)
 
-    def process_single_file(self, path_to_file: Path, row: pd.Series) -> Optional[DataInstance]:
-        start_date = row.export_end_date - timedelta(days=NUM_TIMESTEPS * DAYS_PER_TIMESTEP)
+    def process_single_file(
+        self,
+        path_to_file: Path,
+        row: pd.Series,
+        num_timesteps: int = DEFAULT_NUM_TIMESTEPS,
+    ) -> Optional[DataInstance]:
+        start_date = row.export_end_date - timedelta(days=num_timesteps * DAYS_PER_TIMESTEP)
         da, average_slope = self.load_tif(path_to_file, start_date=start_date)
         closest_lon = self.find_nearest(da.x, row[RequiredColumns.LON])
         closest_lat = self.find_nearest(da.y, row[RequiredColumns.LAT])
@@ -461,7 +489,7 @@ class Engineer:
         labelled_np = self.calculate_ndvi(labelled_np)
         labelled_np = self.remove_bands(labelled_np)
 
-        labelled_array = self.fillna(labelled_np, average_slope)
+        labelled_array = self.fillna(labelled_np, average_slope=average_slope)
         if labelled_array is None:
             return None
 
@@ -488,7 +516,9 @@ class Engineer:
                 print(f"No downloaded files for {region_identifier}")
                 continue
             for region_idx, filepath in enumerate(all_region_files):
-                instance_name, test_instance = self.process_test_file(filepath, region_idx)
+                instance_name, test_instance = self.process_test_file_with_region(
+                    filepath, region_idx
+                )
                 if test_instance is not None:
                     hf = h5py.File(self.test_savedir / f"{instance_name}.h5", "w")
 
